#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from pygoo import BaseObject
from smewt.base import Metadata



class Movie(Metadata):

    typename = 'Movie'

    schema = { 'title': unicode,
               'year': int,
               # more to come
               }

    valid = [ 'title' ]

    unique = [ 'title', 'year' ]

    order = [ 'title', 'year' ]

    converters = {}


class Comment(BaseObject):
    schema = { 'metadata': Metadata,
               'author': unicode,
               'text': unicode,
               'date': int
               }

    reverse_lookup = { 'metadata': 'comments' }

    valid = [ 'metadata', 'author', 'date' ]
    unique = [ 'metadata', 'author', 'date' ]
