/* $Id: example.css,v 1.5 2006/03/27 02:44:36 pat Exp $ */

/*--------------------------------------------------
  REQUIRED to hide the non-active tab content.
  But do not hide them in the print stylesheet!
  --------------------------------------------------*/
.tabberlive .tabbertabhide {
 display:none;
}

/*--------------------------------------------------
  .tabber = before the tabber interface is set up
  .tabberlive = after the tabber interface is set up
  --------------------------------------------------*/
.tabber {
}
.tabberlive {
 margin-top:1em;
}

/**** smewt style rules *****************************************/
/**** for all movies view                                      **/
#wrapper {
 text-align: left;
 margin: 0px auto;
 padding: 0px;
 border:0;
 width: 100%;
}

#header {
 margin: 0 0 10px 0;
 background-color: #F0F0F0;
 padding-top: 10px;
 padding-bottom: 10px;
 text-align: center;
 font: bold 18px Verdana, sans-serif;
 color: #333333;
}

#left-side {
 float: left;
 width: 50%;
}

#right-side {
 margin: 0;
 float: left;
 width: 50%;
 height: 1% /* Holly hack for Peekaboo Bug */
}

.movie {
 height: 90px;
 border: 1px solid #778;
 background-color: #F0F0F0;
 margin: 5px;
 margin-left: 0px;
}

.movie img {
 position: relative;
 top: 5px;
 left: 5px;
 height: 80px;
 float: left;
}

.movie a {
 text-decoration: none;
 font: bold 24px Verdana, sans-serif;
 color: #448;
 margin-left: 30px;
 position: relative;
 top: 30px;
}

/**** end of smewt style rules **********************************/


/**** smewt style rules *****************************************/
/**** for single movie view                                    **/
img {
 position: absolute;
 top: 10px;
 left: 10px;
}

h1 {
 font: bold 36px Verdana, sans-serif;
 margin-left: 0px;
 margin-bottom: 10px;
 margin-top: 40px;
 color: #333333;
}

.rightshifted {
 margin-left: 150px;
 margin-bottom: 50px;
}

.rightshifted a {
 font: 13px Verdana, sans-serif;
 text-decoration: none;
 border: 1px solid #778;
 padding-left: 5px;
 padding-right: 5px;
 padding-top: 2px;
 padding-bottom: 2px;
 background: #DDE;
 color: #448;
}

.rightshifted img {
  position: relative;
  left: 0px;
  top: 0px;
}

.rightshifted a:hover {
 color: #000;
 background: #AAE;
 border-color: #227;
}

.tabberlive .tabbertab {
 color: #7F7F7F;
 font: 16px Verdana, sans-serif;
 margin-left: 3px;
 border: 1px solid #778;
 margin-bottom: 5px;
 padding: 10px;
 background-color: #F0F0F0;
}

.tabberlive .tabbertab p {
 color: #000000;
 padding: 0px;
 margin: 0px;
 margin-top: 10px;
}


.tabberlive .tabbertab a {
 text-decoration: none;
 font-weight: bold;
 color: #448;
}

.tabberlive .tabbertab a:hover {
 text-decoration: underline;
}

.tabberlive .tabbertab .extras {
 margin: 20px;
 font: bold 20px Verdana, sans-serif;
 color: #333333;
}

/**** end of smewt style rules **********************************/

/**** smewt style rules *****************************************/
/**** for recent movies view                                   **/
.commentbox {
    border: 1px solid #778;
    background-color: #F0F0F0;
    margin: 5px;
    margin-left: 0px;
}

.commentbox img {
 position: relative;
 top: 5px;
 left: 5px;
 height: 80px;
 float: left;
}

.commentbox a {
 text-decoration: none;
 font: bold 24px Verdana, sans-serif;
 color: #448;
 margin-left: 30px;
 margin-bottom: 30px;
 position: relative;
 top: 30px;
}

.comments {
    margin-top: 70px;
    margin-left: 20px;
}

.comment {
    margin-left: 20px;
}

.comment pre {
    font: 14px Verdana, sans-serif;
    color: #000000;
}

textarea {
    width: 90%;
}

.singlefile {
 font: 16px Verdana, sans-serif;
 margin-left: 3px;
 border: 1px solid #778;
 margin-bottom: 5px;
 padding: 10px;
 background-color: #F0F0F0;
}

.singlefile p {
 color: #000000;
 padding: 0px;
 margin: 0px;
 margin-top: 10px;
}


.singlefile a {
 text-decoration: none;
 font-weight: bold;
 color: #448;
}

.singlefile a:hover {
 text-decoration: underline;
}


/**** end of smewt style rules **********************************/

/*--------------------------------------------------
  ul.tabbernav = the tab navigation list
  li.tabberactive = the active tab
  --------------------------------------------------*/
ul.tabbernav
{
 margin:0;
 padding: 3px 0;
 border-bottom: 1px solid #778;
 font: bold 12px Verdana, sans-serif;
}

ul.tabbernav li
{
 list-style: none;
 margin: 0;
 display: inline;
}

ul.tabbernav li a
{
 padding: 3px 0.5em;
 margin-left: 3px;
 border: 1px solid #778;
 border-bottom: none;
 background: #DDE;
 text-decoration: none;
}

ul.tabbernav li a:link { color: #448; }
ul.tabbernav li a:visited { color: #667; }

ul.tabbernav li a:hover
{
 color: #000;
 background: #AAE;
 border-color: #227;
}

ul.tabbernav li.tabberactive a
{
 background-color: #fff;
 border-bottom: 1px solid #fff;
}

ul.tabbernav li.tabberactive a:hover
{
 color: #000;
 background: white;
 border-bottom: 1px solid white;
}

/*--------------------------------------------------
  .tabbertab = the tab content
  Add style only after the tabber interface is set up (.tabberlive)
  --------------------------------------------------*/
.tabberlive .tabbertab {
 padding:5px;
 border:1px solid #aaa;
 border-top:0;

 /* If you don't want the tab size changing whenever a tab is changed
    you can set a fixed height */

 /* height:200px; */

 /* If you set a fix height set overflow to auto and you will get a
    scrollbar when necessary */

 /* overflow:auto; */
}

/* If desired, hide the heading since a heading is provided by the tab */
.tabberlive .tabbertab h2 {
 display:none;
}
.tabberlive .tabbertab h3 {
 display:none;
}

/* Example of using an ID to set different styles for the tabs on the page */
.tabberlive#tab1 {
}
.tabberlive#tab2 {
}
.tabberlive#tab2 .tabbertab {
 height:200px;
 overflow:auto;
}

